// CalibrationDialog.cpp : implementation file
//

#include "stdafx.h"
#include "rootfly.h"
#include "CalibrationDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CalibrationDialog dialog


CalibrationDialog::CalibrationDialog(const CString& fname, float conversion, CWnd* pParent /*=NULL*/)
	: CDialog(CalibrationDialog::IDD, pParent),
    m_waiting_for_second(false), m_point1(-1,-1), m_point2(-1,-1), 
    m_pixels_per_mm(conversion), m_border(5)
{
	//{{AFX_DATA_INIT(CalibrationDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
  bool success = blepo::Load(fname, &m_img);
  if (!success)
  {
    CString foo;
    foo.Format("Unable to load file '%s'", (const char*) fname);
    AfxMessageBox(foo);
  }
}


void CalibrationDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CalibrationDialog)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CalibrationDialog, CDialog)
	//{{AFX_MSG_MAP(CalibrationDialog)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_EN_KILLFOCUS(IDC_EDIT_MM, OnKillfocusEditMm)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CalibrationDialog message handlers

void CalibrationDialog::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	// TODO: Add your message handler code here
  Redraw(dc);
	
	// Do not call CDialog::OnPaint() for painting messages
}

void CalibrationDialog::Recalculate()
{
  CString str;
  GetDlgItem(IDC_EDIT_MM)->GetWindowText(str);
  float mm = atof(str);
  str.Format("%5.1f", mm);
  GetDlgItem(IDC_EDIT_MM)->SetWindowText(str);
  GetDlgItem(IDC_EDIT_PIXELS)->GetWindowText(str);
  float pixels = atof(str);
  m_pixels_per_mm = pixels / mm;
  str.Format("%5.1f", m_pixels_per_mm);
  GetDlgItem(IDC_EDIT_PIXELS_PER_MM)->SetWindowText(str);
}

void CalibrationDialog::OnLButtonDown(UINT nFlags, CPoint point) 
{
  CRect rect(m_border, m_border, m_img.Width()+m_border, m_img.Height()+m_border);
  if (rect.PtInRect(point))
  {
    if (!m_waiting_for_second)
    {
      m_point1 = point;
      m_point2 = point;
      m_waiting_for_second = true;
    }
    else
    {
      float dx = m_point1.x - m_point2.x;
      float dy = m_point1.y - m_point2.y;
      float pixels = (float) (sqrt(dx * dx + dy * dy) + 0.5f);
      CString str;
      str.Format("%5.1f", pixels);
      GetDlgItem(IDC_EDIT_PIXELS)->SetWindowText(str);
      Recalculate();
      m_waiting_for_second = false;
    }
  }

	CDialog::OnLButtonDown(nFlags, point);
}

void CalibrationDialog::Redraw(CDC& dc)
{
  blepo::Draw(m_img, dc, m_border, m_border);
  if (m_point1.x >= 0)
  {
    CPen pen(PS_SOLID, 1, RGB(255,0,0));
    CPen* old = dc.SelectObject(&pen);
    dc.MoveTo(m_point1);
    dc.LineTo(m_point2);
    dc.SelectObject(old);
  }
}

void CalibrationDialog::OnMouseMove(UINT nFlags, CPoint point) 
{
  CClientDC dc(this);
  CRect rect(m_border, m_border, m_img.Width()+m_border, m_img.Height()+m_border);
  if (rect.PtInRect(point))
  {
    if (m_waiting_for_second)  
    {
      m_point2 = point;
      Redraw(dc);
    }
  }
  
	CDialog::OnMouseMove(nFlags, point);
}

BOOL CalibrationDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

  float mm = 1;  // default

  float pixels = mm * m_pixels_per_mm;
  CPoint center(m_img.Width()/2, m_img.Height()/2);
  int p = (int) (pixels / 2 + 0.5f);
  m_point1 = CPoint(center.x - p, center.y);
  m_point2 = CPoint(center.x + (pixels - p), center.y);

  CString str;
  str.Format("%5.1f", pixels);
  GetDlgItem(IDC_EDIT_PIXELS)->SetWindowText(str);
  str.Format("%5.1f", mm);
  GetDlgItem(IDC_EDIT_MM)->SetWindowText(str);
  str.Format("%5.1f", m_pixels_per_mm);
  GetDlgItem(IDC_EDIT_PIXELS_PER_MM)->SetWindowText(str);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CalibrationDialog::OnOK() 
{
	// TODO: Add extra validation here
  if (GetFocus() == GetDlgItem(IDC_EDIT_MM))
  {
    Recalculate();
    return;
  }
	
	CDialog::OnOK();
}

void CalibrationDialog::OnKillfocusEditMm() 
{
  Recalculate();	
}
